from django.urls import path

from helpme.views import helpme as views

app_name = "helpme"

urlpatterns = [
    path("", views.SupportDashboardView.as_view(), name="dashboard"),
    path("faq/", views.FAQView.as_view(), name="faq"),
    path("request/", views.SupportRequestView.as_view(), name="submit-request"),
]



# Generated by Django 3.1 on 2020-09-08 22:29

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('helpme', '0004_auto_20200901_2323'),
    ]

    operations = [
        migrations.AddField(
            model_name='comment',
            name='comment_type',
            field=models.IntegerField(choices=[(0, 'Message'), (1, 'History')], default=0, verbose_name='Type'),
        ),
        migrations.AlterField(
            model_name='ticket',
            name='description',
            field=models.TextField(verbose_name='Message'),
        ),
    ]



from django.views.generic.edit import CreateView, UpdateView
from django.urls import reverse_lazy
from django.contrib.auth.mixins import LoginRequiredMixin, PermissionRequiredMixin
from django.contrib.sites.models import Site

from helpme.models import Ticket, Comment, Team, VisibilityChoices, StatusChoices, CommentTypeChoices
from helpme.forms import TicketForm, UpdateTicketForm, CommentForm, QuestionForm, CategoryForm, TeamForm
from .helpme import FAQView, SupportDashboardView


class FAQCreateView(PermissionRequiredMixin, FAQView):
    template_name = "helpme/faq_create.html"
    permission_required = ["helpme.add_question", "helpme.add_category"]
    
    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['question_form'] = QuestionForm
        context['category_form'] = CategoryForm
        return context
    

class AdminSupportDashboardView(PermissionRequiredMixin, SupportDashboardView):
    permission_required = "helpme.see_support_tickets"
    template_name = "helpme/admin_ticket_list.html"
    

class TicketDetailView(LoginRequiredMixin, PermissionRequiredMixin, UpdateView):
    model = Ticket
    form_class = UpdateTicketForm
    template_name = "helpme/ticket_detail.html"
    permission_required = "helpme.see_support_tickets"

    def get_success_url(self):
        return reverse_lazy('helpme_admin:ticket-detail', args=[self.object.uuid])
    
    def get_object(self, queryset=None):
        if queryset is None:
            queryset = self.get_queryset()

        uuid = self.kwargs.get('uuid')

        try:
            obj = queryset.get(uuid=uuid)
        except queryset.model.DoesNotExist:
            raise Http404(_("No %(verbose_name)s found matching the query") %
                          {'verbose_name': queryset.model._meta.verbose_name})
        return obj

    def sort_comments(self):
        ticket = self.get_object()
        if 'oldest' in self.request.GET:
            return ticket.comments.all().order_by('created')
        else:
            return ticket.comments.all().order_by('-created')

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        support = self.request.user.has_perm('helpme.see_support_tickets')
        developer = self.request.user.has_perm('helpme.see_developer_tickets')
        admin = self.request.user.has_perm('helpme.see_all_tickets')
        
        context['support'] = support
        context['developer'] = developer
        context['admin'] = admin

        context['user'] = self.request.user
        context['comment_form'] = CommentForm(support=support)
        context['ticket_form'] = TicketForm
        
        # determines whether status color is red or green
        context['negative_status'] = [StatusChoices.CLOSED, StatusChoices.CANCELED, StatusChoices.HOLD]

        # filter comments by visibility
        if admin:
            comments = self.object.comments.all()
        else:
            comments = self.object.comments.filter(visibility=VisibilityChoices.REPORTERS)
            if support:
                comments |=  self.object.comments.filter(visibility=VisibilityChoices.SUPPORT)
            if developer:
                comments |= self.object.comments.filter(visibility=VisibilityChoices.DEVELOPERS)
        context['comments'] = comments
        context['ticket_comments'] = self.sort_comments
        return context

    def form_valid(self, form):
        action = " updated the "
        cd = form.changed_data
        length = len(cd)
        for field in cd:
            if length > 1:
                if field == cd[-1]:
                    action += "and " + field + " fields"
                else:
                    if length == 2:
                        action += field + " "
                    else:
                        action += field + ", "
            else:
                action += field + " field"    
        Comment.objects.create(content=action, user=self.request.user, ticket=self.get_object(), comment_type=CommentTypeChoices.EVENT, visibility=VisibilityChoices.SUPPORT)
        
        response = super().form_valid(form)
        return response


class TeamCreateView(LoginRequiredMixin, PermissionRequiredMixin, CreateView):
    permission_required = 'helpme.view_team'
    model = Team
    form_class = TeamForm
    success_url = reverse_lazy('helpme_admin:team-list')

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['admin'] = self.request.user.has_perm('helpme.add_team')
        context['user_teams'] = self.request.user.team_set.all()
        context['teams'] = Team.objects.filter(sites__in=[Site.objects.get_current()])
        return context
    

    def form_valid(self, form):
        response = super().form_valid(form)
        form.instance.sites.add(Site.objects.get_current())
        return response
    
    
class TeamDetailView(LoginRequiredMixin, PermissionRequiredMixin, UpdateView):
    permission_required = 'helpme.view_team'
    model = Team
    form_class = TeamForm
    template_name = "helpme/team_detail.html"

    def get_success_url(self):
        return reverse_lazy('helpme_admin:team-detail', args=[self.object.uuid])

    def get_object(self, queryset=None):
        if queryset is None:
            queryset = self.get_queryset()

        uuid = self.kwargs.get('uuid')

        try:
            obj = queryset.get(uuid=uuid)
        except queryset.model.DoesNotExist:
            raise Http404(_("No %(verbose_name)s found matching the query") %
                          {'verbose_name': queryset.model._meta.verbose_name})
        return obj


    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['admin'] = self.request.user.has_perm('helpme.change_team')
        return context



# Generated by Django 3.1 on 2020-08-31 23:43

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('sites', '0002_alter_domain_unique'),
        ('helpme', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Category',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('category', models.CharField(max_length=120)),
                ('localization', models.JSONField(default=dict)),
                ('sites', models.ManyToManyField(to='sites.Site')),
            ],
        ),
        migrations.CreateModel(
            name='Question',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('question', models.CharField(max_length=120)),
                ('answer', models.CharField(max_length=255)),
                ('localization', models.JSONField(default=dict)),
                ('category', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='questions', to='helpme.category')),
                ('sites', models.ManyToManyField(blank=True, to='sites.Site')),
            ],
        ),
    ]



# Generated by Django 3.1 on 2020-09-11 00:10

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('sites', '0002_alter_domain_unique'),
        ('helpme', '0005_auto_20200908_2229'),
    ]

    operations = [
        migrations.RenameField(
            model_name='category',
            old_name='excluded_sites',
            new_name='category_excluded_sites',
        ),
        migrations.RenameField(
            model_name='category',
            old_name='sites',
            new_name='category_sites',
        ),
        migrations.RemoveField(
            model_name='ticket',
            name='history',
        ),
        migrations.AlterField(
            model_name='team',
            name='global_team',
            field=models.BooleanField(default=False),
        ),
        migrations.AlterField(
            model_name='team',
            name='sites',
            field=models.ManyToManyField(to='sites.Site'),
        ),
    ]



from django.urls import path

from helpme.api import views

urlpatterns = [
    path('ticket/', views.CreateTicketAPIView.as_view(), name='helpme-api-create-ticket'),
    path('<uuid:uuid>/comment/', views.CreateCommentAPIView.as_view(), name='helpme-api-create-comment'),
    path('category/', views.CreateCategoryAPIView.as_view(), name='helpme-api-create-category'),
    path('question/', views.CreateQuestionAPIView.as_view(), name='helpme-api-create-question'),
]



from django.forms import ModelForm
from django import forms
from django.utils.translation import gettext as _
from django.contrib.sites.models import Site
from django.contrib.auth import get_user_model

from helpme.models import Ticket, Comment, Category, Question, Team, VisibilityChoices


class TicketForm(ModelForm):

    class Meta:
        model = Ticket
        fields = ['category', 'subject', 'description']

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields['subject'].widget.attrs.update({'placeholder': _('Enter Subject Title')})
        self.fields['description'].widget.attrs.update({'placeholder': _('Enter Details')})


class UpdateTicketForm(ModelForm):
    teams = forms.ModelMultipleChoiceField(queryset=Team.objects.all(), widget=forms.CheckboxSelectMultiple, required=False)

    class Meta:
        model = Ticket
        fields = ['status', 'priority', 'category', 'teams', 'assigned_to', 'dev_ticket', 'question']

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields['dev_ticket'].label = _("Developer Ticket")
        self.fields['question'].label = _("Related Frequently Asked Question")
        
        # filter teams by those that are responsible for the site where the ticket was created
        self.fields['teams'].queryset = Team.objects.filter(sites__in=[self.instance.site])


class CommentForm(ModelForm):
    content = forms.CharField(label=_("Message"), widget=forms.Textarea(attrs={'placeholder': _('Enter a message'), 'rows': '4'}))
    visibility = forms.ChoiceField(label=_("Visibility (Who can see this message)"), choices=VisibilityChoices.choices)
    
    class Meta:
        model = Comment
        fields = ['content', 'visibility']

    def __init__(self, support=True, *args, **kwargs):
        super().__init__(*args, **kwargs)
        if not support:
            self.fields.pop('visibility')
            

class CategoryForm(ModelForm):

    class Meta:
        model = Category
        fields = ['category']


class QuestionForm(ModelForm):

    class Meta:
        model = Question
        fields = ['question', 'answer', 'category']

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        # limit the possible categories to the current site
        current_site = Site.objects.get_current()
        self.fields['category'].queryset = Category.objects.filter(category_sites__in=[current_site])


class TeamForm(ModelForm):
    # get users with explicit support permissions
    # as well as admins with all permissions
    member_qs = get_user_model().objects.filter(user_permissions__codename="see_support_tickets") | get_user_model().objects.filter(is_superuser=True)
    members = forms.ModelMultipleChoiceField(queryset=member_qs.distinct(), widget=forms.CheckboxSelectMultiple)

    class Meta:
        model = Team
        fields = ['name', 'categories', 'members']
